/*
 * MMXGst.cpp
 *
 *  Created on: Aug 20, 2013
 *      Author: mmxbuild
 */

#include "MMXGst.h"
#include <gst/gst.h>
#include <glib.h>
#include <glib-object.h>
#include <iostream>
#include <sstream>
#include <string>
#include <stdio.h>
#include <vector>
#include <iterator>


MMXGst::MMXGst(std::string name) : cmdLine(""), name("Media-slave" + name + ": "), running(true), stream(NULL) {
	std::cout << this->name << " started, listening for command" << std::endl;

}

MMXGst::~MMXGst() {
	delete stream;
}

void MMXGst::start() {
	using namespace std;
	int cmdCount = 0;
	while(running) {

		getline(cin,cmdLine);
		if(cmdLine.find_first_not_of(' ') != std::string::npos)
		{
			cout << name << " Got command " << cmdCount++ << ": " << cmdLine << std::endl;
			istringstream iss(cmdLine);
			vector<string> tokens;
			copy(istream_iterator<string>(iss),
					 istream_iterator<string>(),
					 back_inserter<vector<string> >(tokens));
			handleCommand(tokens);
		}
	}
}

void MMXGst::stop() {
	stream->stop();
}

void MMXGst::setup(std::vector<std::string> &tokens) {

	if(tokens.size() == 6) {
		std::string dstIp = tokens[1];
		int dstPort;
		std::istringstream(tokens[2]) >> dstPort;
		int srcPort;
		std::istringstream(tokens[3]) >> srcPort;
		int payload;
		std::istringstream(tokens[4]) >> payload;
		Payload::Type payloadType = Payload::fromString(tokens[5]);
		if(payloadType != Payload::unknown) {
			stream = new MMXGstStream(name, dstIp,dstPort,srcPort,payload,payloadType);
		} else {
			std::cout << "ERROR: unknown codec. Not starting a stream!" << std::endl;
		}

	} else {
		std::cout << "Setup: Wrong number of arguments!";
	}
}

void MMXGst::play(std::vector<std::string> &tokens) {
	if(tokens.size() == 2 && stream) {
		stream->play(tokens[1]);
		std::cout << name << " Stream is playing file: " << tokens[1] << std::endl;
	} else {
		std::cout << name << " Play: Wrong number of arguments!";
	}
}

void MMXGst::handleCommand(std::vector<std::string> &tokens) {


	if(tokens[0].compare("stop") == 0) {
		stop();
	} else if(tokens[0].compare("setup") == 0) {
		setup(tokens);
	} else if(tokens[0].compare("play") == 0) {
		play(tokens);
	} else if(tokens[0].compare("exit") == 0) {
		running = false;
	} else {
		std::cout << "Unknown command: " << tokens[0] << std::endl;
	}
}


int main(int argc, char *argv[])
{
	std::string name = "unnamed";
	if(argc > 1) {
		name = argv[1];
	}
	MMXGst mmxGst = MMXGst(name);
	mmxGst.start();

//	GMainLoop *loop;
//
//	GstElement *pipeline, *source, *demuxer, *decoder, *conv, *sink;
//	GstBus *bus;
//	guint bus_watch_id;
//
//	/* Initialisation */
//	gst_init(&argc, &argv);
//
//	loop = g_main_loop_new(NULL, FALSE);
//
//
//	/* Check input arguments */
//	if(argc != 2) {
//		g_printerr("Usage: %s <Ogg/Vorbis filename>\n", argv[0]);
//		return -1;
//	}
//
//
//	/* Create gstreamer elements */
//	pipeline = gst_pipeline_new("audio-player");
//	source   = gst_element_factory_make("filesrc",       "file-source");
//	demuxer  = gst_element_factory_make("oggdemux",      "ogg-demuxer");
//	decoder  = gst_element_factory_make("vorbisdec",     "vorbis-decoder");
//	conv     = gst_element_factory_make("audioconvert",  "converter");
//	sink     = gst_element_factory_make("autoaudiosink", "audio-output");
//
//	if(!pipeline || !source || !demuxer || !decoder || !conv || !sink) {
//		g_printerr("One element could not be created. Exiting.\n");
//		return -1;
//	}
//
//	/* Set up the pipeline */
//
//	/* we set the input filename to the source element */
//	g_object_set(G_OBJECT(source), "location", argv[1], NULL);
//
//	/* we add a message handler */
//	bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));
//	bus_watch_id = gst_bus_add_watch(bus, bus_call, loop);
//	gst_object_unref(bus);
//
//	/* we add all elements into the pipeline */
//	/* file-source | ogg-demuxer | vorbis-decoder | converter | alsa-output */
//	gst_bin_add_many(GST_BIN(pipeline), source, demuxer, decoder, conv, sink, NULL);
//
//	/* we link the elements together */
//	/* file-source -> ogg-demuxer ~> vorbis-decoder -> converter -> alsa-output */
//	gst_element_link(source, demuxer);
//	gst_element_link_many(decoder, conv, sink, NULL);
//	g_signal_connect(demuxer, "pad-added", G_CALLBACK(on_pad_added), decoder);
//
//	/* note that the demuxer will be linked to the decoder dynamically.
//     The reason is that Ogg may contain various streams(for example
//     audio and video). The source pad(s) will be created at run time,
//     by the demuxer when it detects the amount and nature of streams.
//     Therefore we connect a callback function which will be executed
//     when the "pad-added" is emitted.*/
//
//
//	/* Set the pipeline to "playing" state*/
//	g_print("Now playing: %s\n", argv[1]);
//	gst_element_set_state(pipeline, GST_STATE_PLAYING);
//
//
//	/* Iterate */
//	g_print("Running...\n");
//	g_main_loop_run(loop);
//
//
//	/* Out of the main loop, clean up nicely */
//	g_print("Returned, stopping playback\n");
//	gst_element_set_state(pipeline, GST_STATE_NULL);
//
//	g_print("Deleting pipeline\n");
//	gst_object_unref(GST_OBJECT(pipeline));
//	g_source_remove(bus_watch_id);
//	g_main_loop_unref(loop);
//
//	return 0;
}
