/*
 * MMXGstStream.cpp
 *
 *  Created on: Aug 20, 2013
 *      Author: mmxbuild
 */

#include "MMXGstStream.h"
#include <iostream>
#include <gst/gst.h>
#include <glib.h>


static void on_pad_added(GstElement *element, GstPad *pad, gpointer data)
{
	GstPad *sinkpad;
	GstElement *decoder =(GstElement *) data;

	g_print("Dynamic pad created, linking demuxer/decoder\n");
	sinkpad = gst_element_get_static_pad(decoder, "sink");

	gst_pad_link(pad, sinkpad);

	gst_object_unref(sinkpad);
}


static gboolean bus_call(GstBus* bus, GstMessage* msg, gpointer data)
{
	GMainLoop* loop =(GMainLoop*) data;

	switch(GST_MESSAGE_TYPE(msg)) {

	case GST_MESSAGE_EOS:
		std::cout << "EOS!" << std::endl;
		g_main_loop_quit(loop);
		break;

	case GST_MESSAGE_ERROR: {
		gchar* debug;
		GError* error;

		gst_message_parse_error(msg, &error, &debug);
		g_free(debug);

		g_printerr("Error: %s\n", error->message);
		std::cout << "Error: " << error->message << std::endl;
		g_error_free(error);

		g_main_loop_quit(loop);
		break;
	}
	default:
		break;
	}

	return TRUE;
}


MMXGstStream::MMXGstStream(std::string name, std::string dstIp, int dstPort, int srcPort, int payload, Payload::Type payloadType) :
	name(name), dstIp(dstIp), dstPort(dstPort), srcPort(srcPort), payload(payload), payloadType(payloadType), setupSuccess(false)
{
	std::cout << name << "Created stream: dstIp=" << dstIp << " dstPort=" << dstPort << " srcPort=" << srcPort << " payload=" << payload << std::endl;
	gst_init(0,0);
	loop = g_main_loop_new(NULL, FALSE);
	pipeline = gst_pipeline_new("player");
	source   = gst_element_factory_make("filesrc", "file-source");

//	GList* list = gst_registry_get_feature_list(gst_registry_get(), GST_TYPE_ELEMENT_FACTORY);
//
//	g_list_foreach(list, (GFunc)print, NULL);



	if(payloadType==Payload::g711a) {
		g_object_set(source, "do-timestamp", true, NULL);
		if(!source)
			std::cout << name << " ERROR: source element could not be created. " << std::endl;
		demuxer  = gst_element_factory_make("decodebin", "parser/demuxer");
		encoder  = gst_element_factory_make("alawenc", "encoder");
		payloader     = gst_element_factory_make("rtppcmapay", "payloader");
	} else if(payloadType==Payload::g711u) {
		g_object_set(source, "do-timestamp", true, NULL);
		if(!source)
			std::cout << name << " ERROR: source element could not be created. " << std::endl;
		demuxer  = gst_element_factory_make("decodebin", "parser/demuxer");
		encoder  = gst_element_factory_make("mulawenc", "encoder");
		payloader     = gst_element_factory_make("rtppcmupay", "payloader");
	} else if(payloadType==Payload::g722) {
		if(!source)
			std::cout << name << " ERROR: source element could not be created. " << std::endl;
		g_object_set(source, "do-timestamp", true, NULL);
		demuxer  = gst_element_factory_make("decodebin", "parser/demuxer");
		encoder  = gst_element_factory_make("avenc_g722", "encoder");
		payloader     = gst_element_factory_make("rtpg722pay", "payloader");
	} else if(payloadType==Payload::h263) {
		demuxer  = gst_element_factory_make("decodebin", "parser/demuxer");
		encoder  = gst_element_factory_make("avenc_h263", "encoder");
		payloader     = gst_element_factory_make("rtph263pay", "payloader");
		if(!encoder)
			std::cout << name << " ERROR: encoder element could not be created. " << std::endl;
		g_object_set(encoder, "rtp-payload-size", 1, NULL);
		g_object_set(payloader, "modea-only", true, NULL);
	} else if(payloadType==Payload::h264) {
		demuxer  = gst_element_factory_make("decodebin", "parser/demuxer");
		encoder  = gst_element_factory_make("x264enc", "encoder");
		payloader     = gst_element_factory_make("rtph264pay", "payloader");
		if(!payloader)
			std::cout << name << " ERROR: payloader element could not be created. " << std::endl;
		g_object_set(payloader, "config-interval", 1, NULL);
	} else {
		std::cout << name << " Wrong payload type!\n" << std::endl;
	}

	sink = gst_element_factory_make("udpsink", "sink");

	if(!pipeline)
		std::cout << name << " ERROR: pipeline element could not be created. " << std::endl;
	if(!source)
		std::cout << name << " ERROR: source element could not be created. " << std::endl;
	if(!demuxer)
		std::cout << name << " ERROR: demuxer element could not be created. " << std::endl;
	if(!encoder)
		std::cout << name << " ERROR: encoder element could not be created. " << std::endl;
	if(!payloader)
		std::cout << name << " ERROR: payloader element could not be created. " << std::endl;
	if(!sink)
		std::cout << name << " ERROR: sink element could not be created. " << std::endl;


	if(pipeline && source && demuxer && encoder && payloader && sink) {
		// Config elements!
		g_object_set(payloader, "pt", payload, NULL);
		g_object_set(sink, "host", dstIp.c_str(), NULL);
		g_object_set(sink, "port", dstPort, NULL);
		g_object_set(sink, "bind-port", srcPort, NULL);
		g_object_set(sink, "async", false, NULL);

		g_signal_connect(G_OBJECT(demuxer), "pad-added", G_CALLBACK(on_pad_added), encoder);

		bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));
		bus_watch_id = gst_bus_add_watch(bus, bus_call, loop);
		gst_object_unref(bus);

		gst_bin_add_many(GST_BIN(pipeline), source, demuxer, encoder, payloader, sink, NULL);
		//gst_element_link(source, demuxer);
		gst_element_link_pads (source, "src", demuxer, "sink");
		gst_element_link_many(encoder, payloader, sink, NULL);
		setupSuccess = true;
		std::cout << "Stream is setup!";
	}

}



MMXGstStream::~MMXGstStream() {
	gst_element_set_state(pipeline, GST_STATE_NULL);
	gst_object_unref(GST_OBJECT(pipeline));
	g_source_remove(bus_watch_id);
	g_main_loop_unref(loop);
	gst_deinit();
}


void* start_loop(GMainLoop* loop) {
	static int i = 0;
	std::cout << "Started thread" << ++i << std::endl;
	g_main_loop_run(loop);
	std::cout << "Ended thread" << --i << std::endl;
	return 0;
}

void MMXGstStream::play(std::string file) {

	if(setupSuccess) {

		gst_element_set_state(pipeline, GST_STATE_READY);
		g_object_set(G_OBJECT(source), "location", file.c_str(), NULL);
		gst_element_set_state(pipeline, GST_STATE_PLAYING);
		if(!g_main_loop_is_running(loop)) {
			if(g_thread_new("nesse", (GThreadFunc)start_loop, (void*)loop) == NULL) {
				g_printerr("Error when creating thread!!");
			}
		}
			//g_main_loop_run(loop);
	}

}

void MMXGstStream::stop() {
	gst_element_set_state(pipeline, GST_STATE_READY);
	std::cout << "EOS!" << std::endl;
}






